#!/usr/bin/python
# -*- coding: UTF-8 -*-

##__author__      = "Jiaxuan Li"
##__email__       = "Kim_lijiaxuan@163.com"
##__date__        = "2020-04-20"

import argparse
from Bio import SeqIO,AlignIO
import glob,os

parser = argparse.ArgumentParser(prog = "script1_Longest ORFs extraction.py",
                                 usage = "%(prog)s [-h] <XXX.transdecoder.cds>",
                                 description = '''
Each of transcript may be identified multiple ORFs of different lengths.
Script1 can extarct the longest ORFs from each of transcript.
Before using it, the users need to perform the ORF identification step (using Transdecoder software as an example).
The output file "XXX.transdecoder.cds" in the ORF identification step is regarded as a input file in script1.''',
                                 epilog="Please contact Jiaxuan Li at Kim_lijiaxuan@163.com if you encounter any problems.",
                                 formatter_class=argparse.RawDescriptionHelpFormatter)

parser.add_argument("-c", required=True, metavar="<filename>", help="input identified coding sequences filename in the FASTA format, required")
args = parser.parse_args()


CDSFileDict = {}
for record in SeqIO.parse(args.c,'fasta'):
    UniqueCDSID = record.description.split(' ')[0] 
    CDSFileDict[UniqueCDSID] = record
    
UniqueIDlist = []
for record in SeqIO.parse(args.c,'fasta'):
    UniqueCDSID = record.description.split(' ')[0]
    if len(UniqueIDlist) == 0:
        UniqueIDlist.append(UniqueCDSID)
    else:            
        if UniqueCDSID.split('|')[0] != UniqueIDlist[-1].split('|')[0]:
            UniqueIDlist.append(UniqueCDSID)
        else:
            if len(record) >= len(CDSFileDict[UniqueIDlist[-1]].seq):
                UniqueIDlist[-1] = UniqueCDSID

out = open('Longest_ORFs_'+str(len(UniqueIDlist))+'.fasta','w')
for seqID in  UniqueIDlist:
    out.write('>' + seqID + ' ' + CDSFileDict[seqID].description.split(' ')[-4] + '\n' +\
              str(CDSFileDict[seqID].seq) + '\n')
out.close()



    


