#!/usr/bin/python
# -*- coding: UTF-8 -*-

##__author__      = "Jiaxuan Li"
##__email__       = "Kim_lijiaxuan@163.com"
##__date__        = "2020-04-20"

import argparse
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC

parser = argparse.ArgumentParser(prog = "script2_reference UTRs generation.py",
                                 usage = "%(prog)s [-h] -c <filename> -a <filename> -d <filename> -l <int>",
                                 description = '''
For a given CDS sequence file, based on a annotation gff3 file, script2 can help users extract 5' UTR and 3'UTR sequences from assembled transcripts.
''',
                                 epilog="Please contact Jiaxuan Li at Kim_lijiaxuan@163.com if you encounter any problems.",
                                 formatter_class=argparse.RawDescriptionHelpFormatter)

parser.add_argument("-c",required=True, metavar="<filename>", help=" input assembled transcript sequence filename in FASTA format, required")
parser.add_argument("-a",required=True, metavar="<filename>", help="input assembled transcript annotation filename in GFF3 format, required")
parser.add_argument("-d",required=True, metavar="<filename>", help="input defined reference coding sequence filename in FASTA format, required")
parser.add_argument("-l",required=True, metavar="<int>", type=int, help="retained UTR length threshold, required")
args = parser.parse_args()


DefinedCDSIDList = []
for record in SeqIO.parse(args.d,'fasta'):
    DefinedCDSIDList.append(record.description.split(' ')[0])#format:c10038_g1_i1|m.17180

FilteredAnnotationInfoDict = {}
AnnotationFileInput = open(args.a,'r')
AnnotationFileLines = AnnotationFileInput.readlines()
for line in AnnotationFileLines:
    if 'UTR' in line:
        info_list = line.split('\n')[0].split('\t')
        geneID = info_list[-1].split('ID=')[1].split('.utr')[0]
        feature = info_list[2]
        start = info_list[3]
        end = info_list[4]
        frame = info_list[6]            
        if geneID in DefinedCDSIDList:
            FilteredAnnotationInfoDict[geneID + ' ' + feature] = geneID +'\t'+ feature +'\t'+ start +\
                                                 '\t'+ end +'\t'+ frame +'\n' #c10038_g1_i1|m.17180

TranscriptContigDict = {}
for record in SeqIO.parse(args.c,'fasta'):
    TranscriptContigDict[record.description.split(' ')[0]] = record #c10038_g1_i1

seqinfoList = []
for key in FilteredAnnotationInfoDict:
    infoList = FilteredAnnotationInfoDict[key].split('\n')[0].split('\t')
    ID = infoList[0]
    seq_feature = infoList[1]
    seq_start = int(infoList[2]) - 1
    seq_end = int(infoList[3])
    seq_Length = seq_end - seq_start
    seq_frame = infoList[-1]
    if ID in DefinedCDSIDList:
        if seq_Length >= args.l:
            if seq_frame == '+':
                seqinfoList.append('>'+ ID + '_'+ seq_feature +'_'+ str(seq_Length) +'\n'+ \
                                   str(TranscriptContigDict[ID.split('|')[0]].seq[seq_start:seq_end])+ '\n')
            else:
                seqinfoList.append('>'+ ID + '_'+ seq_feature +'_'+ str(seq_Length) +'\n'+ \
                                   str(TranscriptContigDict[ID.split('|')[0]].seq[seq_start:seq_end].reverse_complement())+ '\n')

out = open('reference_UTR_lenCutOff'+ str(args.l) + '_' + str(len(seqinfoList)) + '.fasta','w')
for seq in seqinfoList:
    out.write(seq)
out.close()
    
    

