#!/usr/bin/python
# -*- coding: UTF-8 -*-

##__author__      = "Jiaxuan Li"
##__email__       = "Kim_lijiaxuan@163.com"
##__date__        = "2020-04-20"

from __future__ import division
from Bio.Blast import NCBIXML
from Bio import SeqIO,SearchIO,AlignIO
from Bio.Alphabet import IUPAC
from multiprocessing import Manager, Process
import pandas as pd
import numpy as np
import os, shutil
import re
import time
import glob,sys, getopt
import argparse
import multiprocessing


parser = argparse.ArgumentParser(usage = "python Extract_coding_sequences.py -r <filename> -c <contig_dir> -e <exonerate_dir> -o <out_dir> [options]",
                                 description = """

This main task of our script is to analyze the exonerate output file for extracting identified exons and map these exons onto reference to generate coding sequences.
In addition, this script can also generate orthologous coding sequence groups used for subsequent phylogenetic analysis.
Please read README.md for more information.""",
                                 epilog="Please contact Jiaxuan Li at Kim_lijiaxuan@163.com if you encounter any problems.",
                                 formatter_class=argparse.RawDescriptionHelpFormatter)

parser.add_argument("-r",required=True, metavar="<filename>", help="input reference nucleotide sequences filename in the FASTA format, required")
parser.add_argument("-c",required=True, metavar="<contig_dir>", help="input directory for assembled contigs/scaffolds files, required")
parser.add_argument("-e",required=True, metavar="<exonerate_dir>", help="input directory for exonerate output files, required")
parser.add_argument("-o",required=True, metavar="<output_dir>", help="output directory to store all the resulting files, required")
parser.add_argument("-aI", type=float, metavar="<float>", help="amino acid identity (between exon and reference) threshold , default 0.5", default=0.5)
parser.add_argument("-t", type=int, metavar="<int>",help="number of threads, default 1", default=1)
parser.add_argument("-k", type=int, metavar="<int>",choices=['0', '1'], help="1 or 0, default 1, by default, intermediate files will be kept", default=1)
args = parser.parse_args()


def ExtractSeq(exonerateFile,ContigDic,RefDic):

    outputDic = args.o + str(time.strftime('_%Y-%m-%d',time.localtime(time.time())))
    temp_out = os.path.join(outputDic,"temp")
    path_out1 = os.path.join(outputDic,"1.Original_exonerate_analysis_result")
    path_out2 = os.path.join(outputDic,"2.Filtered_exonerate_analysis_result")
    path_out3 = os.path.join(outputDic,"3.Sort_sequence_by_sample")
  
    if os.path.isfile(exonerateFile):
        taxa = re.match("[^_|\.]+",os.path.basename(exonerateFile)).group()
        print taxa,'being processed ...'
        open_exonerateFile = open(exonerateFile,"r")
        lines = open_exonerateFile.readlines()
        Gene_part_number_list = []
        ExonerateResult_Dic = {}
        count = 0
        
        for l_number in range(0,len(lines)):
            if "C4 Alignment:" in lines[l_number]:
                Gene_part_number_list.append(l_number)
        Gene_part_number_list.append(len(lines))
                
        for gpnl_number in range(0,len(Gene_part_number_list)-1):
            gpnl_number2 = gpnl_number+1
            line_start = Gene_part_number_list[gpnl_number]
            line_end = Gene_part_number_list[gpnl_number2]
            Align_info = lines[line_start:line_end]

            Content_List = []
            count += 1
            for ai_number in range(0,len(Align_info)):

                if "Query: " in Align_info[ai_number]:
                    RefID = Align_info[ai_number].split("Query: ")[1].split("\n")[0]#.split(" ")[0] Ptyas_Probe095_ORF_c26475_g1_i2|m.27 type:3prime_partial
                    RawScore = Align_info[ai_number+3].split("Raw score: ")[1].split("\n")[0]
                    RefRange = Align_info[ai_number+4].split("Query range: ")[1]
                    Ref_start = int(RefRange.split(" -> ")[0]) * 3
                    Ref_end = int(RefRange.split(" -> ")[1].split("\n")[0]) * 3
                    Ref_nuc_Len = len(RefDic[RefID].seq)
                    Ref_hitSeq = str(RefDic[RefID].seq[Ref_start:Ref_end])

                    if "revcomp" in Align_info[ai_number+1]:
                        pattern1 = re.compile("\W\Wrevcomp\W")
                        revString = (pattern1.findall(Align_info[ai_number+1]))[0]
                        TargetID = Align_info[ai_number+1].split("Target: ")[1].split(revString)[0].split("\n")[0].split(" ")[0]####NODE_2858_length_1073_cov_24.504016
                        Target_seq_Len = len(ContigDic[taxa][TargetID].seq)
                        TargetRange = Align_info[ai_number+5].split("Target range: ")[1]
                        Target_start = int(TargetRange.split(" -> ")[1].split("\n")[0]) 
                        Target_end = int(TargetRange.split(" -> ")[0])
                        Frame = -1            
                    else:
                        TargetID = Align_info[ai_number+1].split("Target: ")[1].split("\n")[0].split(" ")[0]
                        Target_seq_Len = len(ContigDic[taxa][TargetID].seq)
                        TargetRange = Align_info[ai_number+5].split("Target range: ")[1]
                        Target_start = int(TargetRange.split(" -> ")[0])
                        Target_end = int(TargetRange.split(" -> ")[1].split("\n")[0])
                        Frame = 0
                    Hit_len = int(Target_end) - int(Target_start)
                    Content_List = [RefID,Ref_nuc_Len,TargetID,Target_seq_Len,Ref_start,Ref_end,Target_start,\
                                    Target_end,Hit_len,RawScore,Frame,Ref_hitSeq]


            Identify_list = []
            Target_seq_list = []
            Ref_seq_list = []
            base_list = ["A","G","C","T","-"]
            insert_symbol = ["<","-",">"]

            MSA_info = []               
            for AlignInfo_number in range(0,len(Align_info)):
                if "Target range" in Align_info[AlignInfo_number]:
                    MSA_info.append(AlignInfo_number+2)

                elif "vulgar: " in Align_info[AlignInfo_number]:
                    MSA_info.append(AlignInfo_number)

                else:
                    pass

            MSA_info_start = MSA_info[0]
            MSA_info_end = MSA_info[-1]        
            MSA_info_number = range(MSA_info_start,MSA_info_end)[::5]
            
            for M_number in MSA_info_number:
                RefSeq = list(Align_info[M_number].split(" : ")[1])
                for R_number in range(0,len(RefSeq)):
                    Ref_seq_list.append(RefSeq[R_number])
                    
                TargetSeq = list(Align_info[M_number+3].split(" : ")[1])
                for T_number in range(0,len(TargetSeq)):
                    Target_seq_list.append(TargetSeq[T_number])

                Identify_symbol_pre = list(Align_info[M_number+1].split("\n")[0])
                Identify_symbol_pre_re = Identify_symbol_pre[::-1]
                Identify_symbol_pre_re_filter = Identify_symbol_pre_re[:len(TargetSeq)]
                Identify_symbol_pre = Identify_symbol_pre_re_filter[::-1]                                
                Identify_symbol = "".join(Identify_symbol_pre)
                Identify_list.append(Identify_symbol)             
                    
            Target_HitSeq_filter = []
            Identify_filter = []
            Ref_seq_filter = []
            Target_HitSeq = list("".join(Target_seq_list))
            Identify_total = list("".join(Identify_list))
            Ref_seq_total = list("".join(Ref_seq_list))            
            
            Target_HitSeq_filter = []
            Identify_filter = []
            Ref_seq_filter = []
            Target_HitSeq = list("".join(Target_seq_list))
            Identify_total = list("".join(Identify_list))
            Ref_seq_total = list("".join(Ref_seq_list))                    

            for TH_number in range(0,len(Target_HitSeq)):
                if Target_HitSeq[TH_number] in base_list:
                    Identify_filter.append(Identify_total[TH_number])
                    Target_HitSeq_filter.append(Target_HitSeq[TH_number])
                    Ref_seq_filter.append(Ref_seq_total[TH_number])

            filter_number_list = []
            for rsf_number in range(0,len(Ref_seq_filter)):
                if Ref_seq_filter[rsf_number] not in insert_symbol:
                    filter_number_list.append(rsf_number)

            Target_HitSeq_Final = []
            Identify_Final = []
            Ref_seq_Final = []             

            for fnl_number in filter_number_list:            
                Ref_seq_Final.append(Ref_seq_filter[fnl_number])
                Target_HitSeq_Final.append(Target_HitSeq_filter[fnl_number])
                Identify_Final.append(Identify_filter[fnl_number])
            Identity = float(Identify_Final.count("|")) / float(len(Identify_Final))

            Content_List.append("".join(Target_HitSeq_Final))
            Content_List.append(Identity)
            ExonerateResult_Dic[count] = Content_List
            
        ExonerateResultTable = pd.DataFrame(ExonerateResult_Dic).T
        ExonerateResultTable.columns = ["RefID","Ref_nuc_Len","TargetID","Target_seq_Len","Ref_start","Ref_end",\
                                        "Target_start","Target_end","Hit_len","RawScore","Frame","Ref_hitSeq","Target_hitSeq","AA_identity"]

        ExonerateResultTable_group = ExonerateResultTable.groupby(ExonerateResultTable["RefID"])
        ExonerateResultTable_new = pd.DataFrame(columns=["RefID","Ref_nuc_Len","TargetID","Target_seq_Len","Ref_start","Ref_end",\
                                                         "Target_start","Target_end","Hit_len","RawScore","Frame","Ref_hitSeq",\
                                                         "Target_hitSeq","AA_identity"])

        for key, value in ExonerateResultTable_group:
            sort_value = value.sort_values(by="Ref_start")

            ExonerateResultTable_new = pd.concat([ExonerateResultTable_new, sort_value])
        ExonerateResultTable_new.set_index(["RefID"], inplace=True)        
        ExonerateResultTable_new.to_csv(os.path.join(path_out1,str(taxa)+ "_original_exonerate_analysis_result.csv"))
        #filter
        Identity_filtered_table = ExonerateResultTable_new[ExonerateResultTable_new.AA_identity >= args.aI]        
        Identity_filtered_table.to_csv(os.path.join(temp_out,str(taxa) + "_temp_result_" + str(args.aI) +"identity.txt"),sep="\t")
        open_filteredFile = open(os.path.join(temp_out,str(taxa) + "_temp_result_" + str(args.aI) +"identity.txt"),"r")
        lines = open_filteredFile.readlines()
        LinesDic = {}
        for l_number in range(1,len(lines)):
            infoList = lines[l_number].split("\n")[0].split("\t")
            RefID = infoList[0]
            Hit_len = int(infoList[-6])
            AA_identity = float(infoList[-1])
            Value = float(Hit_len * AA_identity)
            infoList.append(Value)
            LinesDic[l_number] = infoList[:]
        ResultTable = pd.DataFrame(LinesDic).T
        ResultTable.columns = ["RefID","Ref_nuc_Len","TargetID","Target_seq_Len","Ref_start",\
                               "Ref_end","Target_start","Target_end","Hit_len","RawScore","Frame",\
                               "Ref_hitSeq","Target_hitSeq","AA_identity","Value"]

        Value_sort_ResultTable = ResultTable.sort_values(by="Value",ascending=False)#modify sort_values sort_values
        drop_ResultTable = Value_sort_ResultTable.drop_duplicates(subset=["TargetID"], keep="first")
        drop_ResultTable["Ref_start"] = drop_ResultTable["Ref_start"].astype("int")

        drop_ResultTable_group = drop_ResultTable.groupby(drop_ResultTable["RefID"])    
        drop_ResultTable_group_new = pd.DataFrame(columns=["RefID","Ref_nuc_Len","TargetID","Target_seq_Len","Ref_start",\
                                                           "Ref_end","Target_start","Target_end","Hit_len","RawScore","Frame",\
                                                           "Ref_hitSeq","Target_hitSeq","AA_identity","Value"])
                
        for key, value in drop_ResultTable_group:
            sort_value = value.sort_values(by="Ref_start")
            drop_ResultTable_group_new = pd.concat([drop_ResultTable_group_new, sort_value])

        drop_ResultTable_group_new.set_index(["RefID"], inplace=True)
        drop_ResultTable_group_final = drop_ResultTable_group_new.drop(["Value"],axis=1)
        drop_ResultTable_group_final.to_csv(os.path.join(path_out2,str(taxa).split("_")[0] + "_filtered_exonerate_result_"+ \
                                                         str(args.aI) +"AAidentity.txt"),sep="\t")

        #map filtered exons onto reference and extract coding sequences
        open_UniqueFile = open(os.path.join(path_out2,str(taxa).split("_")[0] + "_filtered_exonerate_result_"+ str(args.aI) +\
                                            "AAidentity.txt"),"r")
        unique_lines = open_UniqueFile.readlines()
        unique_RefID_Target_info_Dic = {}
        unique_RefID_record_list = []
        unique_RefID_number_list = []

        for ul_number in range(1,len(unique_lines)):
            unique_RefID = unique_lines[ul_number].split("\r")[0].split("\t")[0]
            if unique_RefID not in unique_RefID_record_list:
                unique_RefID_number_list.append(ul_number)
                unique_RefID_record_list.append(unique_RefID)
        unique_RefID_number_list.append(len(unique_lines))

        Final_seq_File = open(os.path.join(path_out3,str(taxa) + "_ExtractSeq.fasta"),"w")

        for urnl_number in range(0,len(unique_RefID_number_list)-1):
            urnl_number2 = urnl_number + 1
            unique_Ref_info_start = unique_RefID_number_list[urnl_number]
            unique_Ref_info_end = unique_RefID_number_list[urnl_number2]
            unique_Ref_info_list = unique_lines[unique_Ref_info_start:unique_Ref_info_end]

            Final_ref_location = []
            Final_Target_seq = []
            unique_seq_location_List = []
            identity_temp_list = []
            Ref_name = unique_lines[unique_Ref_info_start].split("\n")[0].split("\t")[0]
            Filtered_unique_lines = []

            for uril_number in range(0,len(unique_Ref_info_list)):
                unique_hit_info = unique_Ref_info_list[uril_number].split("\n")[0].split("\t")
                unique_ref_start = int(unique_hit_info[4])
                unique_ref_end = int(unique_hit_info[5])
                unique_seq_location = str(unique_ref_start) +","+ str(unique_ref_end)
                unique_AA_identity = unique_hit_info[-1]
                unique_hit_seq = unique_hit_info[-2]
                identity_temp_list.append(unique_AA_identity)
                unique_seq_location_List.append(unique_seq_location)
                if uril_number== 0:
                    Final_ref_location = [unique_ref_start,unique_ref_end]
                    Filtered_unique_lines.append(unique_Ref_info_list[uril_number])
                else:
                    if int(unique_ref_start) == Final_ref_location[0] and int(unique_ref_end) == Final_ref_location[-1]:

                        if unique_AA_identity >= identity_temp_list[-2]:
                            del(identity_temp_list[-1])
                            Filtered_unique_lines[-1] = unique_Ref_info_list[uril_number]
                            Final_ref_location = [unique_ref_start,unique_ref_end]
                        else:
                            del(identity_temp_list[-1])

                    if int(unique_ref_start) == Final_ref_location[0] and int(unique_ref_end) < Final_ref_location[-1]:

                        if unique_AA_identity >= identity_temp_list[-2]:
                            del(identity_temp_list[-1])
                            Filtered_unique_lines[-1] = unique_Ref_info_list[uril_number]
                            Final_ref_location = [unique_ref_start,unique_ref_end]
                        else:
                            del(identity_temp_list[-1])

                    if int(unique_ref_start) > Final_ref_location[0] and int(unique_ref_end) == Final_ref_location[-1]:

                        if unique_AA_identity >= identity_temp_list[-2]:
                            del(identity_temp_list[-1])
                            Filtered_unique_lines[-1] = unique_Ref_info_list[uril_number]
                            Final_ref_location = [unique_ref_start,unique_ref_end]
                        else:
                            del(identity_temp_list[-1])
                    
                    if int(unique_ref_start) > Final_ref_location[0] and int(unique_ref_start) < Final_ref_location[-1] \
                       and int(unique_ref_end) > Final_ref_location[-1]:         
                        Filtered_unique_lines.append(unique_Ref_info_list[uril_number])
                        Final_ref_location = [unique_ref_start,unique_ref_end]
                    
                    if int(unique_ref_start) == Final_ref_location[-1]:
                        Filtered_unique_lines.append(unique_Ref_info_list[uril_number])
                        Final_ref_location = [unique_ref_start,unique_ref_end]                        
                    
                    if int(unique_ref_start) > Final_ref_location[-1]:
                        Filtered_unique_lines.append(unique_Ref_info_list[uril_number])
                        Final_ref_location = [unique_ref_start,unique_ref_end]
            
            
            unique_AA_identity_temp_list = []
            Final_Ref_location = []
            FinalSeq = []
            Final_Ref_location_record =[]

            for ful_number in range(0,len(Filtered_unique_lines)):
                info = Filtered_unique_lines[ful_number].split("\n")[0].split("\t")
                refStart = int(info[4])
                refEnd = int(info[5])
                unique_AA_identity = info[-1]
                hitSeq = info[-2]
                unique_AA_identity_temp_list.append(unique_AA_identity)
                if ful_number == 0:
                    Final_Ref_location = [refStart,refEnd]
                    FinalSeq.append(hitSeq)
                    Final_Ref_location_record.append(refStart)
                else:
                    #overlap
                    if int(refStart) < Final_Ref_location[-1]:

                        if unique_AA_identity >= unique_AA_identity_temp_list[-2]:

                            if int(refStart) >= Final_Ref_location[0]:
                                Len_Gap = Final_Ref_location[-1] - refStart
                                first_end_location = 0 - Len_Gap
                                temp_unique_hit_seq = FinalSeq[-1][:first_end_location]
                                FinalSeq[-1] = temp_unique_hit_seq
                                FinalSeq.append(hitSeq)
                                Final_Ref_location = [refStart,refEnd]
                            else:
                                FinalSeq[-1] = ""
                                Len_Gap = Final_Ref_location[0] - refStart
                                second_start_location = Len_Gap
                                temp_unique_hit_seq = hitSeq[second_start_location:]
                                FinalSeq.append(temp_unique_hit_seq)
                                Final_Ref_location = [Final_Ref_location[0],refEnd]
                        else:
                            Len_Gap = refEnd - Final_Ref_location[-1]
                            second_start_location = len(hitSeq) - Len_Gap
                            temp_unique_hit_seq = hitSeq[second_start_location:]
                            FinalSeq.append(temp_unique_hit_seq)
                            Final_Ref_location = [Final_Ref_location[-1],refEnd]
                    #no overlap
                    if int(refStart) > Final_Ref_location[-1]:
                        N_gap_number = int(refStart) - Final_Ref_location[-1]
                        temp_unique_hit_seq = "N" * N_gap_number
                        FinalSeq.append(temp_unique_hit_seq)
                        FinalSeq.append(hitSeq)
                        Final_Ref_location = [refStart,refEnd]
                    #connect
                    if int(refStart) == Final_Ref_location[-1]:
                        FinalSeq.append(hitSeq)
                        Final_Ref_location = [refStart,refEnd]

            step = 3
            stopCodon_and_Gap_List = ["TAA","TAG","TGA","---"]
            FinalSeq = "".join(FinalSeq)
            FinalSeq = [FinalSeq[m:m+step] for m in range(0,len(FinalSeq),step)]

            for fs_number in range(0,len(FinalSeq)):
                if FinalSeq[fs_number] in stopCodon_and_Gap_List:
                    FinalSeq[fs_number] = "NNN"
            F_addNs = Final_Ref_location_record[0] * "N"
            Final_seq_File.write(">" + str(taxa) + " "+str(Ref_name) + "\n" + str(F_addNs) + str("".join(FinalSeq))+ "\n")
        Final_seq_File.close()

def mkoutput():
    outputDic = args.o + str(time.strftime('_%Y-%m-%d',time.localtime(time.time())))
    if not os.path.exists(outputDic):
        os.makedirs(outputDic)

    temp_out = os.path.join(outputDic,"temp")
    if not os.path.exists(temp_out):
        os.makedirs(temp_out)

    path_out1 = os.path.join(outputDic,"1.Original_exonerate_analysis_result")
    if not os.path.exists(path_out1):
        os.makedirs(path_out1)

    path_out2 = os.path.join(outputDic,"2.Filtered_exonerate_analysis_result")
    if not os.path.exists(path_out2):
        os.makedirs(path_out2)

    path_out3 = os.path.join(outputDic,"3.Sort_sequence_by_sample")
    if not os.path.exists(path_out3):
        os.makedirs(path_out3)

    path_out4 = os.path.join(outputDic,"4.Alignment")
    if not os.path.exists(path_out4):
        os.makedirs(path_out4)

def runExtractSeq():
    with multiprocessing.Manager() as mg:
        RefDic = mg.dict()
        for ref_seq in SeqIO.parse(args.r,"fasta"):
            RefDic[ref_seq.description] = ref_seq

        ContigDic = mg.dict()
        list1 = os.listdir(args.c)
        for l1_number in range(0,len(list1)):
            seq_handle = os.path.join(args.c,list1[l1_number])
            if os.path.isfile(seq_handle):            
                taxa = re.match("[^_|\.]+",os.path.basename(seq_handle)).group()
                print taxa, 'dict done'
                records = SeqIO.parse(seq_handle,"fasta")
                taxa_Seqdic = {}
                for record in records:
                    key = record.description
                    taxa_Seqdic[key] = record
                ContigDic[taxa] = taxa_Seqdic

        pool = multiprocessing.Pool(processes = args.t)
        list2 = os.listdir(args.e)
        for j in range(0,len(list2)):
            exonerateFiles = os.path.join(args.e,list2[j])
            pool.apply_async(ExtractSeq,(exonerateFiles, ContigDic, RefDic))
        pool.close()
        pool.join()


def temp():
    if args.k == '0':
        os.system("rm -r " + os.path.abspath(temp_out))
    else:
        pass


def GenerateAlignment():    
    outputDic = args.o + str(time.strftime('_%Y-%m-%d',time.localtime(time.time())))
    path_out3 = os.path.join(outputDic,"3.Sort_sequence_by_sample")
    path_out4 = os.path.join(outputDic,"4.Alignment")
    temp_out = os.path.join(outputDic,"temp")    

    path_out5 = os.path.join(temp_out,"temp_alignment")
    if not os.path.exists(path_out5):
        os.makedirs(path_out5)

    referenceName = re.match("[^_|\.]+",os.path.basename(args.r)).group()
    SortDic = {}
    for ref_seq in SeqIO.parse(args.r,"fasta"):
        sortDic_key = ref_seq.description#########
        sortDic_value = ">"+ referenceName + " " + sortDic_key + "\n" + str(ref_seq.seq)+"\n"
        SortDic[sortDic_key] = [sortDic_value]        

    list_path_out3 = os.listdir(path_out3)
    for lpo3_number in range(0,len(list_path_out3)):
        ExtractSeqFile = os.path.join(path_out3,list_path_out3[lpo3_number])
        if os.path.isfile(ExtractSeqFile):
            sample = re.match("[^_|\.]+",os.path.basename(ExtractSeqFile)).group()
            for Seq in SeqIO.parse(ExtractSeqFile, "fasta"):
                #modify 
                Seqid = Seq.description.split(str(sample+" "))[1]
                seq_len = len(Seq.seq)
                ref_len = len(SortDic[Seqid][0].split("\n")[1])
                if Seqid in SortDic:

                    if seq_len <= ref_len:
                        addNnum = ref_len - seq_len
                        taxon_record = ">" + str(Seq.description.split(" ")[0]) + "\n"+str(Seq.seq) + (addNnum)*"N" + "\n"
                        SortDic[Seqid].append(taxon_record)
                    else:
                        pass
                            
    for MSA in SortDic:
        MSAname = re.sub(r'\W','_', MSA)
        MSA_out = open(os.path.join(path_out5,str(MSAname) + ".fasta"),"w")
        for seq in SortDic[MSA]:
            MSA_out.write(seq)
        MSA_out.close()

        
    list_path_out5 = os.listdir(path_out5)
    for lpo5_number in range(0,len(list_path_out5)):
        raw_MSA_File = os.path.join(path_out5,list_path_out5[lpo5_number])
        if os.path.isfile(raw_MSA_File):
            raw_MSA_Filename = os.path.basename(raw_MSA_File).split(".fasta")[0]
            TrimSeq_List = []
            records = SeqIO.parse(raw_MSA_File,"fasta")
            for record in records:
                SeqID = ">" + record.description + "\n"
                TrimSeq_List.append(SeqID)

            Alignment_ori = AlignIO.read(raw_MSA_File,"fasta")
            MSAseqlength = int(Alignment_ori.get_alignment_length())
            align_array = np.array([list(record) for record in Alignment_ori], np.character, order="F")
            New_align_array = np.zeros(shape=(len(align_array),1),dtype="|S1")
            window_size = 12
            align_array = [align_array[:,m:m+window_size] for m in range(0,MSAseqlength,window_size)]

            for sub_array in align_array:
                list1 = sub_array[0]
                times = len(list1)
                for i in range(1,len(sub_array)):
                    list2 = sub_array[i]
                    count = 0
                    for k in zip(list1,list2):
                        if k[1] == "N":
                            count += 1
                        else:
                            if k[0] == k[1]:
                                count += 1
                    identity = float(count)/times
                    if identity <= 0.5:
                        sub_array[i] = "N" * times            
                New_align_array = np.concatenate((New_align_array,sub_array),axis=1)

            for naa_number in range(0,len(New_align_array)):
                TrimSeq_List[naa_number] = TrimSeq_List[naa_number] + "".join(New_align_array[naa_number,:]) + "\n"

            RefinedOutname = os.path.join(path_out4,str(raw_MSA_Filename)+ ".fasta")
            RefinedOut = open(RefinedOutname,"w")
            for Seq in TrimSeq_List:
                RefinedOut.write(Seq)        
            RefinedOut.close()
    os.system("rm -r " + os.path.abspath(path_out5))

               
if __name__ == '__main__':
    print "Step 1: Create output directories..."
    mkoutput()
    print "Step 2: Analyze exonerate output results and extract coding sequences..."
    runExtractSeq()
    print "Step 3: Generate seuqence alignments..."
##    GenerateAlignment()
    temp()
    print 'Done! \n(If each of output directory is empty inside, check that the input file is formatted correctly. The format requirements may refer to our test data. Thanks for use!)'

    
